@@ -10,10 +10,7 @@ import javax.xml.bind.annotation.*;
 import javax.xml.bind.annotation.adapters.*;
 
 import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
-import com.fasterxml.jackson.annotation.JsonInclude;
-import com.fasterxml.jackson.annotation.JsonTypeInfo;
-import com.fasterxml.jackson.annotation.ObjectIdGenerators;
-import com.fasterxml.jackson.annotation.SimpleObjectIdResolver;
+import com.fasterxml.jackson.annotation.*;
 import com.fasterxml.jackson.core.*;
 import com.fasterxml.jackson.databind.*;
 import com.fasterxml.jackson.databind.cfg.MapperConfig;
@@ -72,14 +69,20 @@ public class JaxbAnnotationIntrospector
     extends AnnotationIntrospector
     implements Versioned
 {
-    private static final long serialVersionUID = 2406885758759038380L;
+    private static final long serialVersionUID = -1L;
 
     protected final static String DEFAULT_NAME_FOR_XML_VALUE = "value";
     
     protected final static boolean DEFAULT_IGNORE_XMLIDREF = false;
     
     protected final static String MARKER_FOR_DEFAULT = "##default";
-    
+
+    // @since 2.5
+    protected final static JsonFormat.Value FORMAT_STRING = new JsonFormat.Value().withShape(JsonFormat.Shape.STRING);
+
+    // @since 2.5
+    protected final static JsonFormat.Value FORMAT_INT = new JsonFormat.Value().withShape(JsonFormat.Shape.NUMBER_INT);
+
     protected final String _jaxbPackageName;
     protected final JsonSerializer<?> _dataHandlerSerializer;
     protected final JsonDeserializer<?> _dataHandlerDeserializer;
@@ -89,14 +92,26 @@ public class JaxbAnnotationIntrospector
     protected final boolean _ignoreXmlIDREF;
 
     /**
-     * When using {@link @XmlValue} annotation, a placeholder name is assigned
+     * When using {@link XmlValue} annotation, a placeholder name is assigned
      * to property (unless overridden by explicit name); this configuration
      * value specified what that name is.
      */
     protected String _xmlValueName = DEFAULT_NAME_FOR_XML_VALUE;
 
     /**
-     * @deprecated Since 2.1, use the ctor that takes TypeFactory
+     * Inclusion value to return for properties annotated with 
+     * {@link XmlElement} and {@link XmlElementWrapper}, in case <code>nillable</code>
+     * property is left as <code>false</false>. Default setting is
+     * <code>null</code>; this is typically changed to either
+     * {@link com.fasterxml.jackson.annotation.JsonInclude.Include#NON_NULL}
+     * or {@link com.fasterxml.jackson.annotation.JsonInclude.Include#NON_EMPTY}.
+     *
+     * @since 2.7
+     */
+    protected JsonInclude.Include _nonNillableInclusion = null;
+    
+    /**
+     * @deprecated Since 2.1, use constructor that takes TypeFactory.
      */
     @Deprecated
     public JaxbAnnotationIntrospector() {
@@ -173,7 +188,26 @@ public class JaxbAnnotationIntrospector
     public String getNameUsedForXmlValue() {
         return _xmlValueName;
     }
-    
+
+    /**
+     * Method to call to change inclusion criteria used for property annotated
+     * with {@link XmlElement} or {@link XmlElementWrapper}, with <code>nillable</code>
+     * set as <code>false</code>.
+     *
+     * @since 2.7
+     */
+    public JaxbAnnotationIntrospector setNonNillableInclusion(JsonInclude.Include incl) {
+        _nonNillableInclusion = incl;
+        return this;
+    }
+
+    /**
+     * @since 2.7
+     */
+    public JsonInclude.Include getNonNillableInclusion() {
+        return _nonNillableInclusion;
+    }
+
     /*
     /**********************************************************
     /* Extended API (XmlAnnotationIntrospector)
@@ -272,10 +306,12 @@ public class JaxbAnnotationIntrospector
             }
             switch (m.getParameterCount()) {
             case 0: // getter
-                idPropName = findJaxbPropertyName(m, m.getRawType(), BeanUtil.okNameForGetter(m));
+                idPropName = findJaxbPropertyName(m, m.getRawType(),
+                        BeanUtil.okNameForGetter(m, true));
                 break method_loop;
             case 1: // setter
-                idPropName = findJaxbPropertyName(m, m.getRawType(), BeanUtil.okNameForSetter(m));
+                idPropName = findJaxbPropertyName(m, m.getRawType(),
+                        BeanUtil.okNameForSetter(m, true));
                 break method_loop;
             }
         }
@@ -313,12 +349,15 @@ public class JaxbAnnotationIntrospector
              * is fine...
              */
             if (idref != null) {
+                if (base == null) {
+                    base = ObjectIdInfo.empty();
+                }
                 base = base.withAlwaysAsId(true);
             }
         }
         return base;
     }
-    
+
     /*
     /**********************************************************
     /* General class annotations
@@ -343,16 +382,19 @@ public class JaxbAnnotationIntrospector
     }
     */
 
+    /* 08-Nov-2009, tatus: This is bit trickier: by default JAXB
+     * does actually ignore all unknown properties.
+     * But since there is no annotation to
+     * specify or change this, it seems wrong to claim such setting
+     * is in effect. May need to revisit this issue in future
+     */
+    /*
     @Override
-    public Boolean findIgnoreUnknownProperties(AnnotatedClass ac) {
-        /* 08-Nov-2009, tatus: This is bit trickier: by default JAXB
-         * does actually ignore all unknown properties.
-         * But since there is no annotation to
-         * specify or change this, it seems wrong to claim such setting
-         * is in effect. May need to revisit this issue in future
-         */
-        return null;
-    }
+    public Boolean findIgnoreUnknownProperties(AnnotatedClass ac);
+
+    @Override
+    public JsonIgnoreProperties.Value findPropertyIgnorals(Annotated ac);
+    */
 
     @Override
     public Boolean isIgnorableType(AnnotatedClass ac) {
@@ -371,12 +413,49 @@ public class JaxbAnnotationIntrospector
         return m.getAnnotation(XmlTransient.class) != null;
     }
 
+    //(ryan) JAXB has @XmlAnyAttribute and @XmlAnyElement annotations, but they're not applicable in this case
+    // because JAXB says those annotations are only applicable to methods with specific signatures
+    // that Jackson doesn't support (Jackson's any setter needs 2 arguments, name and value, whereas
+    // JAXB expects use of Map
+
+    // 28-May-2016, tatu: While `@XmlAnyAttribute` looks ALMOST like applicable (esp.
+    //   assuming Jackson could use `Map` field, not just setter/getter), it is alas not.
+    //   The reason is that key is expected to be `QNmae`, XML/JAXB specific name and
+    //   something Jackson does not require or use
+    
+    /*
+    @Override
+    public boolean hasAnySetterAnnotation(AnnotatedMethod am) { }
+    
+    @Override
+    public boolean hasAnySetterAnnotation(AnnotatedMethod am)
+    */
+
+    @Override
+    public Boolean hasRequiredMarker(AnnotatedMember m) {
+        XmlElement elem = m.getAnnotation(XmlElement.class);
+        if ((elem != null) && elem.required()) {
+            return Boolean.TRUE;
+        }
+        XmlAttribute attr = m.getAnnotation(XmlAttribute.class);
+        if ((attr != null) && attr.required()) {
+            return Boolean.TRUE;
+        }
+        // 09-Sep-2015, tatu: Not 100% sure that we should ever return `false`
+        //   here (as it blocks calls to secondary introspector), but since that
+        //   was the existing behavior before 2.6, is retained for now.
+        if ((elem != null) || (attr != null)) {
+            return null;
+        }
+        return Boolean.FALSE;
+    }
+
     @Override
     public PropertyName findWrapperName(Annotated ann)
     {
         XmlElementWrapper w = findAnnotation(XmlElementWrapper.class, ann, false, false, false);
         if (w != null) {
-            /* 18-Sep-2013, tatu: As per #24, need to take special care with empty
+            /* 18-Sep-2013, tatu: As per [jaxb-annotations#24], need to take special care with empty
              *   String, as that should indicate here "use underlying unmodified
              *   property name" (that is, one NOT overridden by @JsonProperty)
              */
@@ -387,9 +466,9 @@ public class JaxbAnnotationIntrospector
                     AnnotatedMethod am = (AnnotatedMethod) ann;
                     String str;
                     if (am.getParameterCount() == 0) {
-                        str = BeanUtil.okNameForGetter(am);
+                        str = BeanUtil.okNameForGetter(am, true);
                     } else {
-                        str = BeanUtil.okNameForSetter(am);
+                        str = BeanUtil.okNameForSetter(am, true);
                     }
                     if (str != null) {
                         return name.withSimpleName(str);
@@ -411,6 +490,26 @@ public class JaxbAnnotationIntrospector
         }
         return null;
     }
+
+    @Override
+    public JsonFormat.Value findFormat(Annotated m) {
+        /* [jaxb-annotations#33]: Use @XmlEnum value (Class) to indicate format,
+         *   iff it makes sense
+         */
+        if (m instanceof AnnotatedClass) {
+            XmlEnum ann = m.getAnnotation(XmlEnum.class);
+            if (ann != null) {
+                Class<?> type = ann.value();
+                if (type == String.class || type.isEnum()) {
+                    return FORMAT_STRING;
+                }
+                if (Number.class.isAssignableFrom(type)) {
+                    return FORMAT_INT;
+                }
+            }
+        }
+        return null;
+    }
     
     /*
     /**********************************************************
@@ -507,8 +606,8 @@ public class JaxbAnnotationIntrospector
         /* First: let's ensure property is a container type: caller should have
          * verified but just to be sure
          */
-        if (!containerType.isContainerType()) {
-            throw new IllegalArgumentException("Must call method with a container type (got "+containerType+")");
+        if (containerType.getContentType() == null) {
+            throw new IllegalArgumentException("Must call method with a container or reference type (got "+containerType+")");
         }
         return _typeResolverFromXmlElements(am);
     }
@@ -555,7 +654,7 @@ public class JaxbAnnotationIntrospector
                     Class<?> refType = elemRef.type();
                     // only good for types other than JAXBElement (which is XML based)
                     if (!JAXBElement.class.isAssignableFrom(refType)) {
-                        // [JACKSON-253] first consider explicit name declaration
+                        // first consider explicit name declaration
                         String name = elemRef.name();
                         if (name == null || MARKER_FOR_DEFAULT.equals(name)) {
                             XmlRootElement rootElement = (XmlRootElement) refType.getAnnotation(XmlRootElement.class);
@@ -572,7 +671,7 @@ public class JaxbAnnotationIntrospector
             }
         }
         
-        // [Issue#1] check @XmlSeeAlso as well.
+        // Check @XmlSeeAlso as well.
         /* 17-Aug-2012, tatu:  But wait! For structured type, what we really is
          *    value (content) type!
          *    If code below does not make full (or any) sense, do not despair -- it
@@ -631,7 +730,7 @@ public class JaxbAnnotationIntrospector
         }
         */
 
-        // [JACKSON-150]: add support for additional core XML types needed by JAXB
+        // Add support for additional core XML types needed by JAXB
         if (type != null) {
             if (_dataHandlerSerializer != null && isDataHandler(type)) {
                 return _dataHandlerSerializer;
@@ -659,38 +758,17 @@ public class JaxbAnnotationIntrospector
     }
 
     @Override
+    @Deprecated // since 2.7
     public Class<?> findSerializationType(Annotated a)
     {
-        // As per [JACKSON-416], need to allow coercing serialization type...
-        /* false for class, package, super-class, since annotation can
-         * only be attached to fields and methods
-         */
-        // Note: caller does necessary sub/supertype checks
-        XmlElement annotation = findAnnotation(XmlElement.class, a, false, false, false);
-        if (annotation == null || annotation.type() == XmlElement.DEFAULT.class) {
-            return null;
+        Class<?> allegedType = _getTypeFromXmlElement(a);
+        if (allegedType != null){
+            Class<?> rawPropType = _rawSerializationType(a);
+            if (!isContainerType(rawPropType)) {
+                return allegedType;
+            }
         }
-        /* [JACKSON-436]: Apparently collection types (array, Collection, maybe Map)
-         *   require type definition to relate to contents, not collection type
-         *   itself. So; we must return null here for those cases, and modify content
-         *   type on another method.
-         */
-        Class<?> rawPropType = _rawSerializationType(a);
-        if (isContainerType(rawPropType)) {
-            return null;
-        }
-        /* [JACKSON-288]: Further, JAXB has peculiar notion of declaring intermediate
-         *  (and, for the most part, useless) type... So basically we better
-         *  just ignore type if there is adapter annotation
-         *  (we could check to see if intermediate type is compatible, but let's not yet
-         *  bother)
-         * 
-         */
-        Class<?> allegedType = annotation.type();
-        if (a.getAnnotation(XmlJavaTypeAdapter.class) != null) {
-            return null;
-        }
-        return allegedType;
+        return null;
     }
 
     /**
@@ -699,22 +777,100 @@ public class JaxbAnnotationIntrospector
      * Jackson defaults (which are configurable), and only using JAXB explicit annotations.
      */
     @Override
+    @Deprecated // since 2.7; remove from 2.8
     public JsonInclude.Include findSerializationInclusion(Annotated a, JsonInclude.Include defValue)
     {
         XmlElementWrapper w = a.getAnnotation(XmlElementWrapper.class);
         if (w != null) {
-            return w.nillable() ? JsonInclude.Include.ALWAYS : JsonInclude.Include.NON_NULL;
+            if (w.nillable()) {
+                return JsonInclude.Include.ALWAYS;
+            }
+            // [jaxb-annotations#52]: Allow specifying inclusion for `nillable=false` too
+            if (_nonNillableInclusion != null) {
+                return _nonNillableInclusion;
+            }
         }
         XmlElement e = a.getAnnotation(XmlElement.class);
         if (e != null) {
-            return e.nillable() ? JsonInclude.Include.ALWAYS : JsonInclude.Include.NON_NULL;
+            if (e.nillable()) {
+                return JsonInclude.Include.ALWAYS;
+            }
+            // [jaxb-annotations#52]: Allow specifying inclusion for `nillable=false` too
+            if (_nonNillableInclusion != null) {
+                return _nonNillableInclusion;
+            }
         }
-        /* [JACKSON-256]: better pass default value through, if no explicit direction indicating
-         * otherwise
-         */
+        //better pass default value through, if no explicit direction indicating otherwise
         return defValue;
     }
-    
+
+    @Override // @since 2.7
+    public JsonInclude.Value findPropertyInclusion(Annotated a)
+    {
+        JsonInclude.Include incl = findSerializationInclusion(a, null);
+        if (incl == null) {
+            return JsonInclude.Value.empty();
+        }
+        return JsonInclude.Value.construct(incl, null);
+    }
+
+    @Override // @since 2.7
+    public JavaType refineSerializationType(final MapperConfig<?> config,
+            final Annotated a, final JavaType baseType) throws JsonMappingException
+    {
+        Class<?> serClass = _getTypeFromXmlElement(a);
+        if (serClass == null) {
+            return baseType;
+        }
+        // First, JAXB has no annotations for key type, so can skip that part (wrt std annotations)
+        // But the meaning of main annotation(s) varies between container/non-container types
+        final TypeFactory tf = config.getTypeFactory();
+        if (baseType.getContentType() == null) { // non-container/-structured types, usually scalar:
+            // 27-Nov-2015, tatu: Since JAXB has just one annotation, must ignore it in
+            //   one direction, typically serialization (but not always):
+            if (!serClass.isAssignableFrom(baseType.getRawClass())) {
+                return baseType;
+            }
+            if (baseType.hasRawClass(serClass)) {
+                // 30-Nov-2015, tatu: As per [databind#1023], need to allow forcing of
+                //    static typing this way
+                return baseType.withStaticTyping();
+            }
+            try {
+                return tf.constructGeneralizedType(baseType, serClass);
+            } catch (IllegalArgumentException iae) {
+                throw new JsonMappingException(null,
+                        String.format("Failed to widen type %s with annotation (value %s), from '%s': %s",
+                                baseType, serClass.getName(), a.getName(), iae.getMessage()),
+                                iae);
+            }
+        } else {
+            // Otherwise, structured type:
+            JavaType contentType = baseType.getContentType();
+            if (contentType != null) { // collection[like], map[like], array, reference
+                // as per earlier, may need to ignore annotation meant for deserialization
+                if (!serClass.isAssignableFrom(contentType.getRawClass())) {
+                    return baseType;
+                }
+                // And then value types for all containers:
+                if (contentType.hasRawClass(serClass)) {
+                     contentType = contentType.withStaticTyping();
+                } else {
+                    try {
+                       contentType = tf.constructGeneralizedType(contentType, serClass);
+                    } catch (IllegalArgumentException iae) {
+                        throw new JsonMappingException(null,
+                                String.format("Failed to widen value type of %s with concrete-type annotation (value %s), from '%s': %s",
+                                        baseType, serClass.getName(), a.getName(), iae.getMessage()),
+                                        iae);
+                    }
+                }
+                return baseType.withContentType(contentType);
+            }
+        }
+        return baseType;
+    }
+
     /*
     /**********************************************************
     /* Serialization: class annotations
@@ -736,12 +892,6 @@ public class JaxbAnnotationIntrospector
         return order;
     }
 
-    @Deprecated // since 2.4
-    @Override
-    public Boolean findSerializationSortAlphabetically(AnnotatedClass ac) {
-        return _findAlpha(ac);
-    }
-
     @Override
     public Boolean findSerializationSortAlphabetically(Annotated ann) {
         return _findAlpha(ann);
@@ -787,28 +937,39 @@ public class JaxbAnnotationIntrospector
     @Override
     public PropertyName findNameForSerialization(Annotated a)
     {
-        // [Issue#69], need bit of delegation (note: copy of super-class functionality)
-        // !!! TODO: in 2.2, remove old methods?
+        // need bit of delegation (note: copy of super-class functionality)
         if (a instanceof AnnotatedMethod) {
             AnnotatedMethod am = (AnnotatedMethod) a;
             if (!isVisible(am)) {
                 return null;
             }
             return findJaxbPropertyName(am, am.getRawType(),
-                    BeanUtil.okNameForGetter(am));
+                    BeanUtil.okNameForGetter(am, true));
         }
         if (a instanceof AnnotatedField) {
             AnnotatedField af = (AnnotatedField) a;
             if (!isVisible(af)) {
                 return null;
             }
+            // Hmmh. As per [jaxb-annotations#61], looks like we must do one more thing to
+            // avoid accidentally exposing transient fields: this is necessary because later on
+            // our returning of "" suggests explicit annotation, which is NOT true, but may be
+            // necessary to indicate that visibility level is satisfied. That works ok except
+            // for specific case of transient fields that we must suppress now
+            if (af.isTransient()) {
+                return null;
+            }
+
             PropertyName name = findJaxbPropertyName(af, af.getRawType(), null);
             /* This may seem wrong, but since JAXB field auto-detection
              * needs to find even non-public fields (if enabled by
              * JAXB access type), we need to return name like so:
              */
+            // 31-Oct-2014, tatu: As per [jaxb-annotations#31], need to be careful to indicate "use default"
+            //    and NOT to force use of specific name; latter would establish explicit name
+            //    and what we want is implicit (unless there is real explicitly annotated name)
             if (name == null) {
-                return new PropertyName(af.getName());
+                return PropertyName.USE_DEFAULT;
             }
             return name;
         }
@@ -823,10 +984,10 @@ public class JaxbAnnotationIntrospector
 
     /**
      *<p>
-     * !!! 12-Oct-2009, tatu: This is hideously slow implementation,
-     *   called potentially for every single enum value being
-     *   serialized. Should improve...
+     * This is very slow implementation, but as of Jackson 2.7, should not be called any more;
+     * instead, {@link #findEnumValues} should be called which has less overhead.
      */
+    @Deprecated // since 2.8, remove from 2.9?
     @Override
     public String findEnumValue(Enum<?> e)
     {
@@ -840,6 +1001,39 @@ public class JaxbAnnotationIntrospector
         }
     }
 
+    @Override // since 2.7
+    public String[] findEnumValues(Class<?> enumType, Enum<?>[] enumValues, String[] names) {
+        HashMap<String,String> expl = null;
+        for (Field f : ClassUtil.getDeclaredFields(enumType)) {
+            if (!f.isEnumConstant()) {
+                continue;
+            }
+            XmlEnumValue enumValue = f.getAnnotation(XmlEnumValue.class);
+            if (enumValue == null) {
+                continue;
+            }
+            String n = enumValue.value();
+            if (n.isEmpty()) {
+                continue;
+            }
+            if (expl == null) {
+                expl = new HashMap<String,String>();
+            }
+            expl.put(f.getName(), n);
+        }
+        // and then stitch them together if and as necessary
+        if (expl != null) {
+            for (int i = 0, end = enumValues.length; i < end; ++i) {
+                String defName = enumValues[i].name();
+                String explValue = expl.get(defName);
+                if (explValue != null) {
+                    names[i] = explValue;
+                }
+            }
+        }
+        return names;
+    }
+
     /*
     /**********************************************************
     /* Deserialization: general annotations
@@ -892,11 +1086,10 @@ public class JaxbAnnotationIntrospector
      * deserialization by using \@XmlElement annotation.
      */
     @Override
+    @Deprecated // since 2.7
     public Class<?> findDeserializationType(Annotated a, JavaType baseType)
     {
-        /* First: only applicable for non-structured types (yes, JAXB annotations
-         * are tricky)
-         */
+        // First: only applicable for non-structured types (yes, JAXB annotations are tricky)
         if (!baseType.isContainerType()) {
             return _doFindDeserializationType(a, baseType);
         }
@@ -906,6 +1099,7 @@ public class JaxbAnnotationIntrospector
     //public Class<?> findDeserializationKeyType(Annotated am, JavaType baseKeyType)
 
     @Override
+    @Deprecated // since 2.7
     public Class<?> findDeserializationContentType(Annotated a, JavaType baseContentType)
     {
         /* 15-Feb-2010, tatus: JAXB usage of XmlElement/XmlElements is really
@@ -937,21 +1131,59 @@ public class JaxbAnnotationIntrospector
                 return type;
             }
         }
-        /* 16-Feb-2010, tatu: May also have annotation associated with field, not method
-         *    itself... and findAnnotation() won't find that (nor property descriptor)
-         */
-        /*
-        if (a instanceof AnnotatedMethod) {
-            AnnotatedMethod am = (AnnotatedMethod) a;
-            // Hmmhh. does name have to match?
-            String propName = am.getName();
-            annotation = findFieldAnnotation(XmlElement.class, am.getDeclaringClass(), propName);
-            if (annotation != null && annotation.type() != XmlElement.DEFAULT.class) {
-                return annotation.type();
+        return null;
+    }
+
+    // @since 2.7
+    @Override
+    public JavaType refineDeserializationType(final MapperConfig<?> config,
+            final Annotated a, final JavaType baseType) throws JsonMappingException
+    {
+        Class<?> deserClass = _getTypeFromXmlElement(a);
+        if (deserClass == null) {
+            return baseType;
+        }
+
+        final TypeFactory tf = config.getTypeFactory();
+
+        if (baseType.getContentType() == null) { // non-container/-structured types, usually scalar:
+            if (baseType.hasRawClass(deserClass)) { // no change
+                return baseType;
+            }
+            // 27-Nov-2015, tatu: Since JAXB has just one annotation, must ignore it in
+            //   one direction, typically serialization (but not always):
+            if (!baseType.getRawClass().isAssignableFrom(deserClass)) {
+                return baseType;
+            }
+            try {
+                return tf.constructSpecializedType(baseType, deserClass);
+            } catch (IllegalArgumentException iae) {
+                throw new JsonMappingException(null,
+                        String.format("Failed to narrow type %s with annotation (value %s), from '%s': %s",
+                                baseType, deserClass.getName(), a.getName(), iae.getMessage()),
+                                iae);
+            }
+        } else {
+            // Otherwise, structured type:
+            JavaType contentType = baseType.getContentType();
+            if (contentType != null) { // collection[like], map[like], array, reference
+                // as per earlier, may need to ignore annotation meant for deserialization
+                if (!contentType.getRawClass().isAssignableFrom(deserClass)) {
+                    return baseType;
+                }
+                // And then value types for all containers:
+                try {
+                   contentType = tf.constructSpecializedType(contentType, deserClass);
+                   return baseType.withContentType(contentType);
+                } catch (IllegalArgumentException iae) {
+                    throw new JsonMappingException(null,
+                            String.format("Failed to narrow type %s with annotation (value %s), from '%s': %s",
+                                    baseType, deserClass.getName(), a.getName(), iae.getMessage()),
+                                    iae);
+                }
             }
         }
-        */
-        return null;
+        return baseType;
     }
 
     /*
@@ -963,7 +1195,7 @@ public class JaxbAnnotationIntrospector
     @Override
     public PropertyName findNameForDeserialization(Annotated a)
     {
-        // [Issue#69], need bit of delegation -- note: copy from super-class
+        // need bit of delegation -- note: copy from super-class
         // TODO: should simplify, messy.
         if (a instanceof AnnotatedMethod) {
             AnnotatedMethod am = (AnnotatedMethod) a;
@@ -971,42 +1203,35 @@ public class JaxbAnnotationIntrospector
                 return null;
             }
             Class<?> rawType = am.getRawParameterType(0);
-            return findJaxbPropertyName(am, rawType, BeanUtil.okNameForSetter(am));
+            return findJaxbPropertyName(am, rawType, BeanUtil.okNameForSetter(am, true));
         }
         if (a instanceof AnnotatedField) {
             AnnotatedField af = (AnnotatedField) a;
+            
             if (!isVisible(af)) {
                 return null;
             }
+            // As per [jaxb-annotations#61] need to do explicit suppress (see
+            // `findNameForSerialization` for details)
+            if (af.isTransient()) {
+                return null;
+            }
+
             PropertyName name = findJaxbPropertyName(af, af.getRawType(), null);
+            
             /* This may seem wrong, but since JAXB field auto-detection
              * needs to find even non-public fields (if enabled by
              * JAXB access type), we need to return name like so:
              */
+            // 31-Oct-2014, tatu: As per [jaxb-annotations#31], need to be careful to indicate "use default"
+            //    and NOT to force use of specific name; latter would establish explicit name
+            //    and what we want is implicit (unless there is real explicitly annotated name)
             if (name == null) {
-                return new PropertyName(af.getName());
+                return PropertyName.USE_DEFAULT;
             }
             return name;
         }
         return null;
-
-        /*
-        if (name != null) {
-            if (name.length() == 0) { // empty String means 'default'
-                return PropertyName.USE_DEFAULT;
-            }
-            return new PropertyName(name);
-        }
-        */
-    }
-
-    @Override
-    public boolean hasAnySetterAnnotation(AnnotatedMethod am) {
-        //(ryan) JAXB has @XmlAnyAttribute and @XmlAnyElement annotations, but they're not applicable in this case
-        // because JAXB says those annotations are only applicable to methods with specific signatures
-        // that Jackson doesn't support (Jackson's any setter needs 2 arguments, name and value, whereas
-        // JAXB expects use of Map
-        return false;
     }
 
     @Override
@@ -1014,12 +1239,6 @@ public class JaxbAnnotationIntrospector
         return false;
     }
 
-    @Override
-    public Boolean hasRequiredMarker(AnnotatedMember m) {
-        XmlElement annotation = m.getAnnotation(XmlElement.class);
-        return annotation == null ? null : Boolean.valueOf(annotation.required());
-    }
-
     @Override
     public Object findDeserializationConverter(Annotated a)
     {
@@ -1045,7 +1264,7 @@ public class JaxbAnnotationIntrospector
         // conversely, here we only apply this to container types:
         Class<?> deserType = _rawDeserializationType(a);
         if (isContainerType(deserType)) {
-            XmlAdapter<?,?> adapter = _findContentAdapter(a, true);
+            XmlAdapter<?,?> adapter = _findContentAdapter(a, false);
             if (adapter != null) {
                 return _converter(adapter, false);
             }
@@ -1205,17 +1424,9 @@ public class JaxbAnnotationIntrospector
         if (element != null) {
             return _combineNames(element.name(), element.namespace(), defaultName);
         }
-        /* 11-Sep-2012, tatu: Wrappers should not be automatically used for renaming.
-         *   At least not here (databinding core can do it if feature enabled).
-         */
-        /*
-        XmlElementWrapper elementWrapper = ae.getAnnotation(XmlElementWrapper.class);
-        if (elementWrapper != null) {
-            return _combineNames(elementWrapper.name(), elementWrapper.namespace(), defaultName);
-        }
-        */
         XmlElementRef elementRef = ae.getAnnotation(XmlElementRef.class);
-        if (elementRef != null) {
+        boolean hasAName = (elementRef != null);
+        if (hasAName) {
             if (!MARKER_FOR_DEFAULT.equals(elementRef.name())) {
                 return _combineNames(elementRef.name(), elementRef.namespace(), defaultName);
             }
@@ -1231,9 +1442,14 @@ public class JaxbAnnotationIntrospector
                 }
             }
         }
+        if (!hasAName) {
+            hasAName = ae.hasAnnotation(XmlElementWrapper.class);
+        }
         // 09-Aug-2014, tatu: Note: prior to 2.4.2, we used to give explicit name "value"
         //   if there was "@XmlValue" annotation; since then, only implicit name.
-        return null;
+
+        // One more thing: 
+        return hasAName ? PropertyName.USE_DEFAULT : null;
     }
 
     private static PropertyName _combineNames(String localName, String namespace,
@@ -1301,8 +1517,9 @@ public class JaxbAnnotationIntrospector
         Class<?> adaptedType = adapterInfo.type();
         
         if (adaptedType == XmlJavaTypeAdapter.DEFAULT.class) {
-            JavaType[] params = _typeFactory.findTypeParameters(adapterInfo.value(), XmlAdapter.class);
-            adaptedType = params[forSerialization ? 1 : 0].getRawClass();
+            JavaType type = _typeFactory.constructType(adapterInfo.value());
+            JavaType[] params = _typeFactory.findTypeParameters(type, XmlAdapter.class);
+            adaptedType = params[1].getRawClass();
         }
         if (adaptedType.isAssignableFrom(typeNeeded)) {
             @SuppressWarnings("rawtypes")
@@ -1387,9 +1604,6 @@ public class JaxbAnnotationIntrospector
         return (n == null) ? null : n.getSimpleName();
     }
 
-    /**
-     * @since 2.1.2
-     */
     protected Class<?> _rawDeserializationType(Annotated a)
     {
         if (a instanceof AnnotatedMethod) {
@@ -1403,9 +1617,6 @@ public class JaxbAnnotationIntrospector
         return a.getRawType();
     }
 
-    /**
-     * @since 2.1.2
-     */
     protected JavaType _fullDeserializationType(AnnotatedMember am)
     {
         if (am instanceof AnnotatedMethod) {
@@ -1413,35 +1624,27 @@ public class JaxbAnnotationIntrospector
             // 27-Nov-2012, tatu: Bit nasty, as we are assuming
             //    things about method signatures here... but has to do
             if (method.getParameterCount() == 1) {
-                return getTypeFactory().constructType(((AnnotatedMethod) am).getGenericParameterType(0),
-                        am.getDeclaringClass());
+                return ((AnnotatedMethod) am).getParameterType(0);
             }
         }
-        return getTypeFactory().constructType(am.getGenericType(),
-                am.getDeclaringClass());
+        return am.getType();
     }
 
-    /**
-     * @since 2.1.2
-     */
     protected Class<?> _rawSerializationType(Annotated a)
     {
         // 27-Nov-2012, tatu: No work-arounds needed yet...
         return a.getRawType();
     }
 
-    /**
-     * @since 2.1.2
-     */
-    protected JavaType _fullSerializationType(AnnotatedMember am)
-    {
-        return getTypeFactory().constructType(am.getGenericType(),
-                am.getDeclaringClass());
+    protected JavaType _fullSerializationType(AnnotatedMember am) {
+        return am.getType();
     }
 
     protected Converter<Object,Object> _converter(XmlAdapter<?,?> adapter, boolean forSerialization)
     {
-        JavaType[] pt = getTypeFactory().findTypeParameters(adapter.getClass(), XmlAdapter.class);
+        TypeFactory tf = getTypeFactory();
+        JavaType adapterType = tf.constructType(adapter.getClass());
+        JavaType[] pt = tf.findTypeParameters(adapterType, XmlAdapter.class);
         // Order of type parameters for Converter is reverse between serializer, deserializer,
         // whereas JAXB just uses single ordering
         if (forSerialization) {
@@ -1449,4 +1652,22 @@ public class JaxbAnnotationIntrospector
         }
         return new AdapterConverter(adapter, pt[0], pt[1], forSerialization);
     }
+
+    protected Class<?> _getTypeFromXmlElement(Annotated a) {
+        XmlElement annotation = findAnnotation(XmlElement.class, a, false, false, false);
+        if (annotation != null) {
+            // Further, JAXB has peculiar notion of declaring intermediate (and, for the
+            // most part, useless) type... So basically we betterjust ignore type if there
+            // is adapter annotation (we could check to see if intermediate type is compatible,
+            // but let's not yet bother)
+            if (a.getAnnotation(XmlJavaTypeAdapter.class) != null) {
+                return null;
+            }
+            Class<?> type = annotation.type();
+            if (type != XmlElement.DEFAULT.class) {
+                return type;
+            }
+        }
+        return null;
+    }
 }
